/*
  xephem-lx50.c - glue code for Meade LX-50's and xephem.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <math.h>
#include <time.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifndef FIFO_NAME
#define FIFO_NAME "/usr/X11R6/lib/X11/xephem/fifos/xephem_in_fifo"
#endif

#ifndef SERIAL_DEV
#define SERIAL_DEV "/dev/cuaa0"
#endif

#ifndef BAUD_RATE
#define BAUD_RATE "300"
#endif

#define GET_RA_COMMAND ":GR#"
#define GET_DEC_COMMAND ":GD#"

static char *fifo_name = FIFO_NAME;
static char *serial_dev = SERIAL_DEV;
static char *baud_rate = BAUD_RATE;

static int to_scope[2];
static int from_scope[2];

float
calc_year()
{
  struct tm* tm;
  time_t t;

  t = time(NULL);
  tm = gmtime(&t);

  return (float)(tm->tm_year + 1900 +
                 (double)tm->tm_yday / 365);
}

char *
read_from_scope(int fd)
{
  static char scope_str[100];
  int i;
  char c;

  i = 0;
  while (read(fd, &c, 1) != -1)
    if (c == '#') break;
    else {
      scope_str[i++] = c;
    }

  scope_str[i++] = 0;

  return scope_str;
}

void
sigchild_handler()
{
  /* child dies.  let's go away. */
  exit(0);
}

void
do_parent()
{
  char get_ra[] = GET_RA_COMMAND;
  char get_dec[] = GET_DEC_COMMAND;
  char *ra;
  char *dec;
  float ra_float, dec_float;
  int i;
  char c;
  int fifo_fd;
  char buf[1024];
  int ra_hour;
  int ra_min;
  int ra_tmin;
  int dec_deg;
  int dec_min;

  fifo_fd = open(fifo_name, O_RDWR);

  close(to_scope[1]);
  close(from_scope[0]);

  /* read the word "Connected.\n" */
  while (read(from_scope[0], &c, 1) != -1)
     if (c == '\n') break;

  while (1)
    {
      write(to_scope[0], get_ra, strlen(get_ra));
      write(to_scope[0], get_dec, strlen(get_dec));

      /* read the ra */
      ra = read_from_scope(from_scope[1]);

      /* convert the RA into radians, in the range of 0..2*pi */
      sscanf(ra, "%d:%d.%d", &ra_hour, &ra_min, &ra_tmin);
      ra_float = (ra_hour * 60.0 + ra_min + ra_tmin / 10.0) / 60.0;
      ra_float = ra_float/24 * 2 * M_PI;

      /* read the dec */
      dec = read_from_scope(from_scope[1]);

      /* convert the DEC into radians, in the range of -pi..pi */
      sscanf(dec, "%d:%d", &dec_deg, &dec_min);
      dec_float = dec_deg + (float)dec_min / 60.0;
      dec_float = dec_float/360 * M_PI;

      snprintf(buf, sizeof(buf), "RA:%2.6f Dec:%2.6f Epoch:%5.3f\n",
               ra_float, dec_float, calc_year());

      write (fifo_fd, buf, strlen(buf));
    }
}

void
do_child()
{
  close(to_scope[0]);
  close(from_scope[1]);

  close(0);
  close(1);
  dup2(to_scope[1], 0);
  dup2(from_scope[0], 1);

  execl(CU_PATH, "cu", "-l", serial_dev, "-s", baud_rate, NULL);
}

void
usage(char *name)
{
  fprintf(stderr, "usage: %s [-l serial device] [-f fifo_name]\n", name);
  fprintf(stderr, "default serial device is %s\n", SERIAL_DEV);
  fprintf(stderr, "default fifo is %s\n", FIFO_NAME);
}

void
error(char *name, char *msg)
{
  fprintf (stderr, "%s error: %s\n", name, msg);

  usage(name);
  exit(1);
}

void
parse_command_line(int argc,
                   char **argv)
{
  int i;

  for (i = 1; i < argc; i ++)
  {
    if (argv[i][0] != '-')
      {
        char buf[1024];
        sprintf(buf, "switch %s not recognized.", argv[i]);
        error (argv[0], buf);
      }

    switch (argv[i][1])
      {
      case 'l':
        if (i == argc - 1)
          {
            error(argv[0], "-l takes argument");
          }
        else
          {
            serial_dev = argv[++i];
          }
        break;
      case 'f':
        if (i == argc - 1)
          {
            error(argv[0], "-f takes argument");
          }
        else
          {
            fifo_name = argv[++i];
          }
        break;
      case '?':
        usage(argv[0]);
        exit(0);
      }
  }
}

int
main(int argc,
     char **argv)
{
  struct stat buf;

  pipe(to_scope);
  pipe(from_scope);

  signal(SIGCHLD, sigchild_handler);

  parse_command_line(argc, argv);

  if (stat(fifo_name, &buf) == -1)
    {
      fprintf(stderr, "can't access fifo %s\n", fifo_name);
      exit(0);
    }
  else if (!S_ISFIFO(buf.st_mode))
    {
      fprintf(stderr, "%s is not a fifo\n");
    }

  switch(fork())
    {
      case -1:
        perror("fork failed");
        break;
      case 0:
        do_child();
        fprintf(stderr, "error: child returned\n");
        break;
     default:
        do_parent();
        break;
    }
}
